# Summary

This gives a very simple app illustrating a problem with the class
loading for record types, centered around the map-> constructors built
by defrecord. It also includes a workaround with a macro defrecord*
that defines a clojure-native map constructor.


# The Problem

    (defrecord Rec ...)
    
    (instance? Rec (Rec. ...))        ;=> true
    (instance? Rec (map->Rec {...}))  ;=> can be false  

This occurs when the code is wrapped in a tomcat servlet by `lein ring
uberwar`, but *not* when run at the REPL or under Jetty, say.

Although produced under ring, this seems to me to be a general problem
with the map-> constructor, as (true? (instance? Rec (map->Rec {...})))
should be an invariant, regardless of the environment or context.

The problem seems to arise in some aspect of the class loading process:

    (.getClassLoader Rec)                    ;=> WebappClassLoader (delegate: false, repositories: /WEB-INF/classes/, parent: org.apache.catalina.loader.StandardClassLoader@790bc49d)
    (.getClassLoader (class (Rec. ...)))     ;=> WebappClassLoader (same as the previous line)
    (.getClassLoader (class (map->Rec ...))) ;=> clojure.lang.DynamicClassLoader@2e7227a8
 
The map->Rec delegates to the create method, which seems to be where the problem lies.


# Environment

Apache Tomcat/6.0.24 JVM/1.6.0_26-b03  Linux 2.6.32-279.el6.x86_64

Clojure 1.4.0,  Ring 1.1.6, Compojure 1.1.3, Lein-Ring Plugin 0.7.5 (for lein ring uberwar)


# Code Organization

+ recordbug.core        -- The code that illustrates the problem
+ recordbug.server      -- Ring server boilerplate
+ recordbug.workaround  -- A wrapper macro defrecord* defining an alternative
                           map constructor, map=>TypeName, without this problem


# Usage

1. Start the server under Jetty and open in a browser with
   
       lein ring server
   
   Alternatively, you can pick a port, say 8080, and do

       lein run 8080

   or 
       lein ring server-headless 8080

   at the command line prompt, and then access the server at
   localhost:8080, replacing 8080 with whatever port you choose.

2. Visit /bug and /workaround to see the results.
   These should show that all of the instance? calls properly return true.
   
3. Make a war file with
   
       lein ring uberwar recordbug.war
   
4. Deploy it via your Tomcat manager.
   
5. Again visit /bug and /workaround.
   
   The instance? calls with the map-> constructor will improperly show false.
   Note the difference in the class loaders.  The clojure-native workaround 
   map constructor, labeled map=> here, gives the correct behavior as it
   just delegates to the positional constructor.



  
