(ns recordbug.server
  (:use compojure.core
        [compojure.route :only [not-found]]
        [compojure.handler :only [api]]
        ring.adapter.jetty
        recordbug.core)
  (:import [recordbug.core Rec WRec]))

(def running-jetty? (atom false))

(defroutes rb-routes
  (GET "/" []
    "<p>This gives a very simple app illustrating a problem with the class
loading for record types, centered around the map-> constructors built
by defrecord. It also illustrates a workaround.</p>
<p>Visit /bug to see the problem and /workaround to test a workaround.</p>
<p>The problem manifests under Tomcat but not under Jetty, so try both.</p>")
  (GET "/bug" []
    (letfn [(outf [r what]
                  (format
                   "<h2>%s</h2>\n<ul>\n<li>Instance?: %s</li><li>Class Loader: %s</li></ul>\n"
                   what (instance? Rec r) (.getClassLoader (class r))))]
      (str
       (if running-jetty?
         "<h1>Running Jetty -- All Instance?'s should be true</h1>"
         "<h1>Running Servlet -- All Instance?'s should be true, but won't be</h1>")
       "\n"
       (format "<h2>Record Type Rec</h2><p>Classloader: %s</p>\n"
               (.getClassLoader Rec))
       (outf (Rec. 2 3 5)                "Direct Constructor Rec.")
       (outf (map->Rec {:a 2 :b 3 :c 5}) "Map Constructor map->Rec"))))
  (GET "/workaround" []
    (letfn [(outf [r what]
                  (format
                   "<h2>%s</h2>\n<ul>\n<li>Instance?: %s</li><li>Class Loader: %s</li></ul>\n"
                   what (instance? WRec r) (.getClassLoader (class r))))]
      (str
       (if running-jetty?
         "<h1>Running Jetty -- All Instance?'s should be true</h1>"
         "<h1>Running Servlet -- All Instance?'s should be true, but won't be</h1>")
       "\n"
       (format "<h2>Record Type Rec</h2><p>Classloader: %s</p>\n"
               (.getClassLoader Rec))
       (outf (WRec. 2 3 5)                "Direct Constructor WRec.")
       (outf (map->WRec {:a 2 :b 3 :c 5}) "Map Constructor map->WRec")
       (outf (map=>WRec {:a 2 :b 3 :c 5}) "Workaround Constructor map=>WRec"))))
  (not-found "The requested resource cannot be found"))

(def rbdemo (api #'rb-routes))

(defn -main [& [default-port]]
  (let [port (Integer. (get (System/getenv) "PORT"
                            (or default-port "8080")))]
    (swap! running-jetty? not)
    (run-jetty rbdemo {:port port})))

